@@ -1,6 +1,6 @@
 /*
  *  Off-the-Record Messaging library
- *  Copyright (C) 2004-2007  Ian Goldberg, Chris Alexander, Nikita Borisov
+ *  Copyright (C) 2004-2008  Ian Goldberg, Chris Alexander, Nikita Borisov
  *                           <otr@cypherpunks.ca>
  *
  *  This library is free software; you can redistribute it and/or
@@ -95,6 +95,8 @@ void otrl_sm_state_new(OtrlSMState *smst)
     smst->pab = NULL;
     smst->qab = NULL;
     smst->nextExpected = OTRL_SMP_EXPECT1;
+    smst->received_question = 0;
+    smst->sm_prog_state = OTRL_SMP_PROG_OK;
 }
 
 /*
@@ -115,6 +117,8 @@ void otrl_sm_state_init(OtrlSMState *smst)
     smst->q = gcry_mpi_new(SM_MOD_LEN_BITS);
     smst->pab = gcry_mpi_new(SM_MOD_LEN_BITS);
     smst->qab = gcry_mpi_new(SM_MOD_LEN_BITS);
+    smst->received_question = 0;
+    smst->sm_prog_state = OTRL_SMP_PROG_OK;
 }
 
 /*
@@ -605,6 +609,7 @@ gcry_error_t otrl_sm_step1(OtrlSMAliceState *astate,
     }
     gcry_mpi_set(astate->secret, secret_mpi);
     gcry_mpi_release(secret_mpi);
+    astate->received_question = 0;
 
     otrl_sm_msg1_init(&msg1);
 
@@ -619,13 +624,14 @@ gcry_error_t otrl_sm_step1(OtrlSMAliceState *astate,
 
     serialize_mpi_array(output, outputlen, SM_MSG1_LEN, msg1);
     otrl_sm_msg_free(&msg1, SM_MSG1_LEN);
+    astate->sm_prog_state = OTRL_SMP_PROG_OK;
     return gcry_error(GPG_ERR_NO_ERROR);
 }
 
 /* Receive the first message in SMP exchange, which was generated by
  * otrl_sm_step1.  Input is saved until the user inputs their secret
  * information.  No output. */
-gcry_error_t otrl_sm_step2a(OtrlSMBobState *bstate, const unsigned char* input, const int inputlen)
+gcry_error_t otrl_sm_step2a(OtrlSMBobState *bstate, const unsigned char* input, const int inputlen, int received_question)
 {
     gcry_mpi_t *msg1;
     gcry_error_t err;
@@ -634,6 +640,8 @@ gcry_error_t otrl_sm_step2a(OtrlSMBobState *bstate, const unsigned char* input,
     if (! bstate->g1) {
 	otrl_sm_state_init(bstate);
     }
+    bstate->received_question = received_question;
+    bstate->sm_prog_state = OTRL_SMP_PROG_CHEATED;
 
     /* Read from input to find the mpis */
     err = unserialize_mpi_array(&msg1, SM_MSG1_LEN, input, inputlen);
@@ -662,6 +670,7 @@ gcry_error_t otrl_sm_step2a(OtrlSMBobState *bstate, const unsigned char* input,
     gcry_mpi_powm(bstate->g2, msg1[0], bstate->x2, SM_MODULUS);
     gcry_mpi_powm(bstate->g3, msg1[3], bstate->x3, SM_MODULUS);
 
+    bstate->sm_prog_state = OTRL_SMP_PROG_OK;
     return gcry_error(GPG_ERR_NO_ERROR);
 }
 
@@ -739,6 +748,7 @@ gcry_error_t otrl_sm_step3(OtrlSMAliceState *astate, const unsigned char* input,
     
     *output = NULL;
     *outputlen = 0;
+    astate->sm_prog_state = OTRL_SMP_PROG_CHEATED;
     
     err = unserialize_mpi_array(&msg2, SM_MSG2_LEN, input, inputlen);
     if (err != gcry_error(GPG_ERR_NO_ERROR)) return err;
@@ -800,6 +810,7 @@ gcry_error_t otrl_sm_step3(OtrlSMAliceState *astate, const unsigned char* input,
     gcry_mpi_release(qa2);
     gcry_mpi_release(inv);
 
+    astate->sm_prog_state = OTRL_SMP_PROG_OK;
     return gcry_error(GPG_ERR_NO_ERROR);
 }
 
@@ -823,6 +834,7 @@ gcry_error_t otrl_sm_step4(OtrlSMBobState *bstate, const unsigned char* input, c
 
     *output = NULL;
     *outputlen = 0;
+    bstate->sm_prog_state = OTRL_SMP_PROG_CHEATED;
     
     if (err != gcry_error(GPG_ERR_NO_ERROR)) return err;
 
@@ -866,6 +878,9 @@ gcry_error_t otrl_sm_step4(OtrlSMBobState *bstate, const unsigned char* input, c
     gcry_mpi_release(rab);
     gcry_mpi_release(inv);
 
+    bstate->sm_prog_state = comp ? OTRL_SMP_PROG_FAILED :
+	OTRL_SMP_PROG_SUCCEEDED;
+
     if (comp)
         return gcry_error(GPG_ERR_INV_VALUE);
     else
@@ -884,6 +899,7 @@ gcry_error_t otrl_sm_step5(OtrlSMAliceState *astate, const unsigned char* input,
     gcry_mpi_t *msg4;
     gcry_error_t err;
     err = unserialize_mpi_array(&msg4, SM_MSG4_LEN, input, inputlen);
+    astate->sm_prog_state = OTRL_SMP_PROG_CHEATED;
     
     if (err != gcry_error(GPG_ERR_NO_ERROR)) return err;
 
@@ -903,6 +919,9 @@ gcry_error_t otrl_sm_step5(OtrlSMAliceState *astate, const unsigned char* input,
     gcry_mpi_release(rab);
     otrl_sm_msg_free(&msg4, SM_MSG4_LEN);
 
+    astate->sm_prog_state = comp ? OTRL_SMP_PROG_FAILED :
+	OTRL_SMP_PROG_SUCCEEDED;
+
     if (comp)
         return gcry_error(GPG_ERR_INV_VALUE);
     else
